/*
Copyright (C) 2009 -  Claudio Salvadori 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

 /**
 *\file CGraph.cpp
 *\brief  Graph class method development.
 */

#include "CGraph.h"
#include <myUtils.h>
#include "TStyle.h"
#include <iostream>
#include <exception> 
#include <sstream>

IRenderer::Factory registerFactoryG("graph", CGraph::createObj);

class GraphException : public std::exception {
public:
        GraphException(const char *msg) throw() : errMessage(msg) { } ;
        ~GraphException(void) throw() { } ;
        const char *what(void) const throw() { return errMessage.c_str(); };
private:
        std::string errMessage;
        GraphException(void) throw();
};

IRenderer* CGraph::createObj(const void* node){
	return new CGraph((const DOMNode*)node);
}

CGraph::CGraph(const DOMNode* node){
	parser_ = new CXMLParser;
	
	//function for graph presets xml parsing 
	graphPreset(node);
	
	counter = 0;
	version = 0;
	
	#ifdef DEBUG
	cerr << "Minimum: " << min_ << endl;
	cerr << "Maximum: " << max_ << endl;
	cerr << "nBins: " << nBins_ << endl;
	#endif
	
	graph_ = new TH1D(name_.c_str(), title_.c_str(), max_, -0.5, max_);			
	
	std::stringstream ss; 
	ss << name_ << "_" << version;
	ss >> nameGraph;
		
	if((cTitle_ != "") && (cWidth_ != 0) && (cHeight_ != 0)){	
		setCanvas();	
	}else{	
		graph_->Draw("LP");
	}
	graph_->SetName(nameGraph.c_str());
}

CGraph::~CGraph(){
	graph_->Write();
	if(graph_) delete graph_;	
	if(canv_) delete canv_;	
	if(gnode_) delete gnode_;
	if(parser_) delete parser_;	
	if(cnode_) delete cnode_;
}

void CGraph::graphPreset(const DOMNode* node){
	char ret[256];
	
	if(!parser_->FindNodeRootat((DOMNode*)node, (char*)"graph_preset", gnode_)){
		throw GraphException("graph_preset node isn't found");
	}else{
		if (!parser_->GetAttribute(gnode_, (char*)"graph_name", (char*)ret)){
			throw GraphException("graph_name attribute isn't found");
		}else{
			#ifdef DEBUG
			printf("Graph name returned: %s\n", ret);
			#endif
			name_ = ret;		/* Graph name*/
			IRenderer::checkName(name_);
		}
				
		if (!parser_->GetAttribute(gnode_, (char*)"graph_title", (char*)ret)){
			throw GraphException("graph_title attribute isn't found");
		}else{
			#ifdef DEBUG
			printf("Graph title returned: %s\n", ret);
			#endif
			title_ = ret;		/* Graph title*/
		}
		
		if (!parser_->GetAttribute(gnode_, (char*)"graph_max", (char*)ret)){
			throw GraphException("graph_max attribute isn't found");
		}else{
			max_ = atof(ret);	/* Graph max value*/
			#ifdef DEBUG
			printf("Graph maximum value returned: %s\n", ret);
			#endif		
		}
	}
	canvasPreset(node);
}

void CGraph::canvasPreset(const DOMNode* node){
	char ret[256];
	if(parser_->FindNodeRootat((DOMNode*)node, (char*)"canvas_preset", cnode_)){		
		if (!parser_->GetAttribute(cnode_, (char*)"canvas_title", (char*)ret)){
			throw GraphException("canvas_title attribute isn't found");
		}else{
			#ifdef DEBUG
			printf("Canvas title: %s\n", ret);
			#endif
			cTitle_ = ret;		/* Canvas title*/
		}
		
		if (!parser_->GetAttribute(cnode_, (char*)"canvas_width", (char*)ret)){
			throw GraphException("canvas_width attribute isn't found");
		}else{
			#ifdef DEBUG
			printf("Canvas width: %s\n", ret);
			#endif
			cWidth_ = atoi(ret);	/* Canvas width (in pixels)*/
		}
		
		if (!parser_->GetAttribute(cnode_, (char*)"canvas_height", (char*)ret)){
			throw GraphException("canvas_height attribute isn't found");
		}else{
			#ifdef DEBUG
			printf("Canvas name: %s\n", ret);
			#endif
			cHeight_ = atoi(ret);	/* Canvas height (in pixels)*/
		}
	}else{
		cTitle_ = "";
		cWidth_ = 0;
		cHeight_ = 0;
	}
	
}

void CGraph::setCanvas(void){
	std::string cName_ = name_;
	cName_.push_back((char)'_');
	
	#ifdef DEBUG
	cout << "Canvas name : " << cName_ << endl;
	#endif

	//Instanciate a canvas
	canv_ = new TCanvas(cName_.c_str(), cTitle_.c_str(), cWidth_, cHeight_);
	
	canv_->SetGrid();
	canv_->SetFillColor(0);
	canv_->SetBorderMode(0);
	canv_->SetBorderSize(2);
	canv_->SetFrameBorderMode(0);
	canv_->SetHighLightColor(0);
	canv_->Draw();
	canv_->Update();
	graph_->Draw("LP");
	canv_->cd();
}

void CGraph::render(void* data, unsigned long size){
	switch(vType_){
		case T_INT8:{
			char dummy = *((char*)data);
			#ifdef DEBUG
			cout << "Data to fill: " << dummy << endl;
			#endif
			fillGraph((double)dummy);
		}
		if(canv_){
			canv_->Modified();
			canv_->Update();
		}
		break;
		case T_UINT8: {
			unsigned char dummy = *((unsigned char*)data);
			#ifdef DEBUG
			cout << "Data to fill: " << dummy << endl;
			#endif
			fillGraph((double)dummy);
		}	
		if(canv_){
			canv_->Modified();
			canv_->Update();
		}
		break;
			
		case T_INT16:{
			short dummy = *((short*)data);
			#ifdef DEBUG
			cout << "Data to fill: " << dummy << endl;
			#endif
			fillGraph((double)dummy);
		}
		if(canv_){
			canv_->Modified();
			canv_->Update();
		}
		break;
			
		case T_UINT16:{
			unsigned short dummy = *((unsigned short*)data);
			#ifdef DEBUG
			cout << "Data to fill: " << dummy << endl;
			#endif
			fillGraph((double)dummy);
		}	
		if(canv_){
			canv_->Modified();
			canv_->Update();
		}
		break;
			
		case T_INT32:{
			int dummy = *((int*)data);
			#ifdef DEBUG
			std::cout << "Data to fill: " << dummy << std::endl;
			#endif
			fillGraph((double)dummy);;
		}
		if(canv_){
			canv_->Modified();
			canv_->Update();
		}
		break;
		
		case T_UINT32:{
			unsigned int dummy = *((unsigned int*)data);
			#ifdef DEBUG
			std::cout << "Data to fill: " << dummy << std::endl;
			#endif
			fillGraph((double)dummy);
		}
		if(canv_){
			canv_->Modified();
			canv_->Update();
		}
		break;
			
		case T_INT64:{
			long dummy = *((long*)data);
			#ifdef DEBUG
			cout << "Data to fill: " << dummy << endl;
			#endif
			fillGraph((double)dummy);
		}
		if(canv_){
			canv_->Modified();
			canv_->Update();
		}
		break;
			
		case T_UINT64:{
			unsigned long dummy = *((unsigned long*)data);
			#ifdef DEBUG
			cout << "Data to fill: " << dummy << endl;
			#endif
			fillGraph((double)dummy);
		}
		if(canv_){
			canv_->Modified();
			canv_->Update();
		}
		break;
		
		case T_FLOAT32:{
			float dummy = *((float*)data);
			#ifdef DEBUG
			cout << "Data to fill: " << dummy << endl;
			#endif
			fillGraph((double)dummy);
		}
		if(canv_){
			canv_->Modified();
			canv_->Update();
		}
		break;
			
		case T_FLOAT64:{
			double dummy = *((double*)data);
			#ifdef DEBUG
			cout << "Data to fill: " << dummy << endl;
			#endif
			fillGraph((double)dummy);
		}
		if(canv_){
			canv_->Modified();
			canv_->Update();
		}
		break;
		
		default:
			break;
	}

}

void CGraph::fillGraph(double data){
	counter++;
	#ifdef DEBUG
	std::cout << "Fill graph: " << data << std::endl;
	#endif
	graph_->SetBinContent(graph_->FindBin(counter), (double)data);
		
	if(counter == (unsigned int)max_){
		counter = 0;
		#ifdef DEBUG
		std::cerr << "Graph_name: " << nameGraph << std::endl;
		#endif
		graph_->Write();
		graph_->GetListOfFunctions()->Delete();
		graph_->Reset("ICEM");
		version ++;
		std::stringstream ss; 
		ss << name_ << "_" << version;
		ss >> nameGraph;
		graph_->SetName(nameGraph.c_str());
	}
}

